#version 430

layout (triangles) in; // tris in
layout (triangle_strip, max_vertices = 90) out; // tris out (max 30 layers)

in VertexData_VS_GS {
	vec3 positionWorld;
	vec3 normalWorld;
	vec2 uvs;
} vertIn[];

out VertexData_GS_FS {
    vec3 normalWorld;
    vec2 uvs;
	flat float normalizedDistance;
} vertOut;

uniform mat4 viewProjectionMat;

uniform int numberOfLayers = 30; // number of fur layers
uniform float totalFurDepth = 0.04; // total depth of fur
uniform vec3 velocity;

void main() {
	vec3 invVelocity = -velocity;

	for (int layer = 0; layer < numberOfLayers; layer++) { // for each layer
		float normalizedDistance = layer / float(numberOfLayers); // root is 0, tip is 1

		for (int v = 0; v < gl_in.length(); v++) { // for each vertex
			vec3 N = normalize(vertIn[v].normalWorld);
			vec3 P = vertIn[v].positionWorld;
			P = P + N * normalizedDistance * totalFurDepth; // ad distance in normal direction
			
			// add gravity:
			float k = pow(normalizedDistance, 3.0);
			P = P + k * vec3(0.0, -1.0, 0.0) * 0.01; // last is gravity strength
			P = P + k * invVelocity * 0.01; // last is velocity strength
			
			vertOut.normalWorld = N;
			vertOut.uvs = vertIn[v].uvs;
			vertOut.normalizedDistance = normalizedDistance;
			gl_Position = viewProjectionMat * vec4(P, 1.0);
			EmitVertex();
		}
		EndPrimitive(); // finish triangle
	}
}
