#version 430 core

layout(location = 0) out vec4 color;
layout(location = 1) out vec4 bloomColor;

in VertexData {
	vec3 positionWorld;
	vec3 normalWorld;
	vec2 uvs;
    float height;
} vert;

uniform sampler2D albedoMap;
uniform float brightness;

uniform vec3 cameraWorld;
uniform vec3 materialCoeff; 
uniform float specularAlpha;

struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointLight;

struct DirectionalLight {
	vec3 color;
	vec3 direction;
} directionalLight;

vec3 phongFunc(vec3 n, vec3 l, vec3 v, vec3 diffC, float diffF, vec3 specC, float specF, float alpha, vec3 attenuation, bool att) {
	float d = length(l);
	l = normalize(l);
	vec3 r = reflect(-l, n);
	float attenuationNew = 1.0;	
	if(att) {attenuationNew = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);}
	return (diffF * diffC * max(0, dot(n, l)) + specF * specC * pow(max(0, dot(r, v)), alpha)) * attenuationNew; 
}

void main() {
	pointLight.color = vec3(1.0, 0.5, 0.5);
    pointLight.position = vec3(1.0, 0.7, -1.0);
    pointLight.attenuation = vec3(1.0, 0.4, 0.1);

    directionalLight.color = vec3(1.0, 0.6, 0.6);
    directionalLight.direction = vec3(-1, -1, -1);

    
    vec3 n = normalize(vert.normalWorld);
	vec3 v = normalize(cameraWorld - vert.positionWorld);
	
	vec3 tex = texture(albedoMap, vert.uvs).rgb;

	color = vec4(tex * materialCoeff.x, 1);
	color.rgb += phongFunc(n, pointLight.position - vert.positionWorld, v, pointLight.color * tex, materialCoeff.y, pointLight.color, materialCoeff.z, specularAlpha, pointLight.attenuation, true);
	color.rgb += phongFunc(n, -directionalLight.direction, v, directionalLight.color * tex, materialCoeff.y, directionalLight.color, materialCoeff.z, specularAlpha, vec3(0), false);
    
    float bright = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722)); // grayscale brigthness of color
    color.rgb += brightness;

    if(bright > 0.3)
        bloomColor = vec4(color.rgb, 1.0) + 0.1;
    else
        bloomColor = vec4(0.0, 0.0, 0.0, 1.0);

}