#version 430

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uvs;

out VertexData {
	vec3 positionWorld;
	vec3 normalWorld;
	vec2 uvs;
    float height;
} vert;

uniform mat4 viewProjectionMat;
uniform mat4 modelObjMat;
uniform mat3 normalMat;

uniform float time;

float displacementFunc(vec3 pos)
{
   return 0.08 * sin(time + pos.x * 2.0) * cos(time * 0.5 + pos.z * 3.5) - 0.9;
}

void main() {
	vert.uvs = vec2(uvs.x, 1.0-uvs.y);
	vec4 pos = modelObjMat * vec4(position, 1);

    vec3 v0 = pos.xyz;
    vec3 v1 = v0 + vec3(0.01, 0.0, 0.0); // used to calculate new normal for v0!
    vec3 v2 = v0 + vec3(0.0, 0.0, 0.01); // -||-

    float d0 = displacementFunc(v0);
    float d1 = displacementFunc(v1);
    float d2 = displacementFunc(v2);

    vert.height = d0;

    v0.xyz += ((d0+1)/2) * normal;
    v1.xyz += ((d1+1)/2) * normal;
    v2.xyz += ((d2+1)/2) * normal;

    vec3 newNormal = normalize(cross(v2 - v0, v1 - v0));

    pos.xyz = v0;
	vert.positionWorld = pos.xyz;

    vert.normalWorld = normalMat * newNormal;

	gl_Position = viewProjectionMat * pos;
}