#version 430 core

layout (points) in; // points in
layout (triangle_strip, max_vertices = 4) out; // quads out

uniform mat4 modelObjMat;
uniform mat4 viewMat;
uniform mat4 projectionMat;

in VertexData {
    float timeToLive;
    float maxTimeToLive;
} vert[]; // array length depends on input layout (length 1 for points)

out vec2 uvs;

flat out float timeToLive; // flat: no interpolation between vertices
flat out float maxTimeToLive;

void main () {
    //const float size = 0.017f * (-pow(2, (1.0 - vert[0].timeToLive/vert[0].maxTimeToLive)) + 2);
    const float size = 0.0006f * (-pow(2, (1.0 - vert[0].timeToLive/vert[0].maxTimeToLive)) + 2);

    mat4 modelViewMat = viewMat * modelObjMat;
    mat4 viewProjectionMat = projectionMat * viewMat;
    
    vec3 right = vec3(modelViewMat[0][0], modelViewMat[1][0], modelViewMat[2][0]); // depandent on camera
    vec3 up = vec3(modelViewMat[0][1], modelViewMat[1][1], modelViewMat[2][1]); // depandent on camera

    vec3 P = gl_in[0].gl_Position.xyz;

    // upper left
    vec3 vb = P - (right - up) * size;
    gl_Position = viewProjectionMat * vec4(vb, 1.0);
    uvs = vec2(0.0, 1.0);
    timeToLive = vert[0].timeToLive;
    maxTimeToLive = vert[0].maxTimeToLive;
    EmitVertex();

    // lower left
    vec3 va = P - (right + up) * size;
    gl_Position = viewProjectionMat * vec4(va, 1.0);
    uvs = vec2(0.0, 0.0);
    timeToLive = vert[0].timeToLive;
    maxTimeToLive = vert[0].maxTimeToLive;
    EmitVertex();
    
    // upper right
    vec3 vc = P + (right + up) * size;
    gl_Position = viewProjectionMat * vec4(vc, 1.0);
    uvs = vec2(1.0, 1.0);
    timeToLive = vert[0].timeToLive;
    maxTimeToLive = vert[0].maxTimeToLive;
    EmitVertex();

    // lower right
    vec3 vd = P + (right - up) * size;
    gl_Position = viewProjectionMat * vec4(vd, 1.0);
    uvs = vec2(1.0, 0.0);
    timeToLive = vert[0].timeToLive;
    maxTimeToLive = vert[0].maxTimeToLive;
    EmitVertex();

    EndPrimitive(); // not needed
}
