#version 430

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uvs;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;

const int SHADOW_MAPS = 2;
const int SPOT_SHADOW_MAPS = 1;

out VertexData {
	vec3 positionWorld;
	vec3 normalWorld;
	vec2 uvs;
	mat3 tbn;
	vec4 frag_pos_lightspace[SHADOW_MAPS];
	vec4 frag_pos_lightspace_spot[SPOT_SHADOW_MAPS];
} vert;

uniform mat4 modelObjMat;
uniform mat4 viewProjectionMat;
uniform mat3 normalMat;
uniform mat4 light_space_matrices[SHADOW_MAPS];
uniform mat4 light_space_matrices_spot[SPOT_SHADOW_MAPS];

void main() {
	// calculate BTN
	vec3 T = normalize(vec3(modelObjMat * vec4(tangent,   0.0)));
	vec3 B = normalize(vec3(modelObjMat * vec4(bitangent, 0.0)));
	vec3 N = normalize(vec3(modelObjMat * vec4(normal,    0.0)));
	vert.tbn = mat3(T, B, N);

	vec4 pos = modelObjMat * vec4(position, 1);
	vert.positionWorld = pos.xyz;
	vert.normalWorld = normalMat * normal;
	vert.uvs = vec2(uvs.x, 1.0-uvs.y);

	for (int i = 0; i < SHADOW_MAPS; i++) {
		vert.frag_pos_lightspace[i] = light_space_matrices[i] * vec4(vert.positionWorld, 1.0);
	}

	for (int i = 0; i < SPOT_SHADOW_MAPS; i++) {
		vert.frag_pos_lightspace_spot[i] = light_space_matrices_spot[i] * vec4(vert.positionWorld, 1.0);
	}
	
	gl_Position = viewProjectionMat * pos;
}
