#version 430 core

layout(location = 0) out vec4 color;

in vec2 outUVs;

uniform sampler2D volumetricTex;
uniform bool horizontal;
uniform float nearPlane;
uniform float farPlane;

const int kernelRadius = 3;

float gaussianWeights[4] = float[4] (
    0.09973557010035818f, // 0
    0.09666702920071231f, // 1
    0.08801633169107488f, // 2
    0.07528435803870111f // 3
);

vec4 biliteralBlur(vec2 direction, vec2 texelSize, int kernelRadius);

void main()
{
    vec2 texelSize = 1.0 / textureSize(volumetricTex, 0);

    if(horizontal) 
    {
        color = biliteralBlur(vec2(0, 1), texelSize, 3);
    }
    else
    {
        color = biliteralBlur(vec2(1, 0), texelSize, 3);
    }
}

float linearEyeDepth(float depth_value) {
	float z = depth_value * 2.0 - 1.0;
    return (2.0 * nearPlane * farPlane) / (farPlane + nearPlane - z * (farPlane - nearPlane));	
}

vec4 biliteralBlur(vec2 direction, vec2 texelSize, int kernelRadius)
{
    vec4 midColor = texture(volumetricTex, outUVs);
    vec3 col = midColor.xyz;

    float midDepth = linearEyeDepth(midColor.w);

    float weightSum = 0;

    float weight = gaussianWeights[0];

    col *= weight;
    weightSum += weight;

    for(int i = -kernelRadius; i <= kernelRadius; i++)
    {
        if (i != 0) 
        {
            vec2 position = outUVs + (direction * i) * texelSize;
            vec4 sampleColor = texture(volumetricTex, position);
            float sampleDepth = linearEyeDepth(sampleColor.w);

            float depthDiff = abs(midDepth - sampleDepth);
            float dfactor = depthDiff * 0.5; // blur depth factor
            float w = exp(-(dfactor * dfactor));

            weight = gaussianWeights[abs(i)] * w;

            col += weight * sampleColor.xyz;
            weightSum += weight;
        }
    }

    col /= weightSum;
    return vec4(col, 0.0);
}