#version 430

layout(location = 0) out vec4 color;
layout(location = 1) out vec4 bloomColor;

in CornerData {
    vec2 center;
	vec2 topLeft;
    vec2 topRight;
    vec2 bottomLeft;
    vec2 bottomRight;
} corner;

uniform sampler2D volumetricTex;
uniform sampler2D scene;
uniform float nearPlane;
uniform float farPlane;

float linearEyeDepth(float depth_value);

void main()
{
    vec4 volumetricTexTopLeft = texture(volumetricTex, corner.topLeft);
	vec4 volumetricTexTopRight = texture(volumetricTex, corner.topRight);
	vec4 volumetricTexBottomLeft = texture(volumetricTex, corner.bottomLeft);
	vec4 volumetricTexBottomRight = texture(volumetricTex, corner.bottomRight);
	
    vec4 sceneColor = texture(scene, corner.center);

	vec4 lowResDepth = vec4(linearEyeDepth(volumetricTexTopLeft.w), 
							linearEyeDepth(volumetricTexTopRight.w), 
							linearEyeDepth(volumetricTexBottomLeft.w), 
							linearEyeDepth(volumetricTexBottomRight.w));
	vec4 highResDepth = vec4(linearEyeDepth(sceneColor.w));

	vec4 depthDiff = abs(lowResDepth - highResDepth);
	float accumulatedDiff = dot(depthDiff, vec4(1));

    vec4 volumetricColor;

    if (accumulatedDiff < 0.5) //upsample threshold
	{ 
		volumetricColor = texture(volumetricTex, corner.center);
	} 
	else 
	{
		float minDepthDiff = depthDiff[0];
		volumetricColor = volumetricTexTopLeft;
		
		if (depthDiff[1] < minDepthDiff) {
			minDepthDiff = depthDiff[1];
			volumetricColor = volumetricTexTopRight;
		}
		
		if (depthDiff[2] < minDepthDiff) {
			minDepthDiff = depthDiff[2];
			volumetricColor = volumetricTexBottomLeft;
		}
		
		if (depthDiff[3] < minDepthDiff) {
			minDepthDiff = depthDiff[3];
			volumetricColor = volumetricTexBottomRight;
		}
	}

    color = volumetricColor + sceneColor;

	//float bright = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
	float bright = dot(color.rgb, vec3(0.5, 0.8, 0.5)); 

    if(bright > 1.0)
        bloomColor = vec4(color.rgb, 1.0);
    else
        bloomColor = vec4(0.0, 0.0, 0.0, 1.0);
}

float linearEyeDepth(float depth_value) {
	float z = depth_value * 2.0 - 1.0;
    return (2.0 * nearPlane * farPlane) / (farPlane + nearPlane - z * (farPlane - nearPlane));	
}